#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
from unittest.mock import MagicMock, call, patch

import pytest
from productfetcher.exceptions import RetryableFetchError
from productfetcher.retry import retry


class FailNTimes:
    def __init__(self, n):
        self.total = n
        self.current = 1

    def __call__(self, *args, **kwargs):
        if self.current <= self.total:
            self.current += 1
            raise RetryableFetchError(f"Failure #{self.current-1}")


def test_fail_n_times():
    x = FailNTimes(2)
    with pytest.raises(RetryableFetchError):
        x()
    with pytest.raises(RetryableFetchError):
        x()
    x()


def test_retrying_succeeds(capsys):
    # two failures is OK
    with patch("time.sleep") as sleep:
        fail_twice = MagicMock(wraps=FailNTimes(2))
        retry(fail_twice, [3, 7])
        assert sleep.call_count == 2
        assert sleep.call_args_list == [call(3), call(7)]
    capsys.readouterr()


def test_retrying_fails(capsys):
    # three failures is too many
    with patch("time.sleep"):
        fail_thrice = MagicMock(wraps=FailNTimes(3))
        with pytest.raises(RetryableFetchError):
            retry(fail_thrice, [3, 7])
    capsys.readouterr()
